try:
    from PyKDE4.kdecore import *
    from PyKDE4.kdeui import *
    from PyKDE4.kio import KDirSelectDialog
    from PyKDE4 import kdecore, kdeui
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *
    from PyQt4 import QtCore, QtGui
except ImportError:
    print("MKV Splitter requires PyQt4 and PyKDE4 in order to work, please make sure these are installed and try to run MKV Splitter again.")
    sys.exit(2)
try:
    from configobj import ConfigObj
except:
    print("It seems the configobj file is missing from your installation. This is strange.")
    sys.exit(2)
try:
    from mydirrequester import *
except ImportError:
    print("mydirrequester seems to be missing from your installation. This is strange.")
    sys.exit(2)  
import os

class ConfigureDialog(QDialog):
    def __init__(self, parent):
        QDialog.__init__(self)
        self.config=self.read_config()
        self.setup_ui()
        
    def setup_ui(self):
        self.main_container = QWidget()
        self.main_layout = QVBoxLayout(self.main_container)
        
        self.form_container = QWidget()
        self.form_layout = QFormLayout(self.form_container)
        self.setLayout(self.main_layout)
        self.main_layout.addWidget(self.form_container)
        
        self.default_close_after_split_checkbox = QCheckBox()
        if str(self.config['default_close_after_split']) == 'True':
            self.default_close_after_split_checkbox.setChecked(True)
        
        self.default_output_directory_is_source_checkbox = QCheckBox()
        if str(self.config['default_output_directory_is_source']) == 'True':
            self.default_output_directory_is_source_checkbox.setChecked(True)
        self.default_output_directory_is_source_checkbox.stateChanged.connect(self.directory_is_source_toggled)    
        
        self.default_output_directory_edit = MyDirRequester()
        self.default_output_directory_edit.setText(self.config['default_output_directory'])
        
        self.form_layout.addRow('Close after split by default?: ', self.default_close_after_split_checkbox)
        self.form_layout.addRow('Default output directory is same as input file?', self.default_output_directory_is_source_checkbox)
        self.form_layout.addRow('Default output directory: ',  self.default_output_directory_edit)
        
        self.button_layout_container = QWidget()
        self.button_layout = QHBoxLayout(self.button_layout_container)
        
        self.ok_button = QPushButton()
        self.ok_button.setText('&Ok')
        self.ok_button.clicked.connect(self.ok_clicked)
        self.ok_icon = self.loadIcon('dialog-ok')
        self.ok_button.setIcon(self.ok_icon)
        
        self.cancel_button = QPushButton()
        self.cancel_button.setText('&Cancel')
        self.cancel_button.clicked.connect(self.cancel_clicked)
        self.cancel_icon = self.loadIcon('dialog-cancel')
        self.cancel_button.setIcon(self.cancel_icon)
        
        self.button_layout.addWidget(self.ok_button)
        self.button_layout.addWidget(self.cancel_button)
        
        self.main_layout.addWidget(self.button_layout_container)
        self.directory_is_source_toggled()
        self.show()
    def directory_is_source_toggled(self):
        if self.default_output_directory_is_source_checkbox.isChecked():
            self.default_output_directory_edit.setEnabled(False)
        else:
            self.default_output_directory_edit.setEnabled(True)
    def ok_clicked(self):
        self.write_config()
        self.accept()
    
    def cancel_clicked(self):
        self.reject()
        
    def loadIcon(self, name):
        k = KIconLoader()
        return QIcon(k.loadIcon(name, KIconLoader.Small))
    
    def read_config(self):
        config = ConfigObj()
        try:
            if os.path.exists('%s/.mkvsplitter.conf' % os.environ['HOME']):
                config = ConfigObj("%s/.mkvsplitter.conf" % os.environ['HOME'])
        except:
            pass
        finally:
            if not 'default_close_after_split' in config:
                config['default_close_after_split'] = False
            if not 'default_output_directory' in config:
                config['default_output_directory'] = os.environ['HOME']
            if not 'default_output_directory_is_source' in config:
                config['default_output_directory_is_source'] = True
            return config
            
    def write_config(self):
        self.config['default_close_after_split'] = self.default_close_after_split_checkbox.isChecked()
        self.config['default_output_directory_is_source'] = self.default_output_directory_is_source_checkbox.isChecked()
        if os.path.isdir(self.default_output_directory_edit.url()):
            self.config['default_output_directory'] = self.default_output_directory_edit.url()
        
        self.config.filename= "%s/.mkvsplitter.conf" % os.environ['HOME']
        self.config.write()
        
