#!/usr/bin/python2
# (c) 2011 Dustin Widmann
# GPL v3
# Small python2 kde4 program to split MKV files based on chapter information.

try:
    import os, getopt, sys, subprocess
    from subprocess import CalledProcessError
except ImportError:
    print('MKV Splitter requires the following standard modules to be present in order to function: os, getopt, sys, subprocess\nIf your version of python2 doesn\'t include these, try upgrading to a more recent version of python2')
    sys.exit(2)

try:
    from PyKDE4.kdecore import *
    from PyKDE4.kdeui import *
    from PyKDE4.kio import KFileDialog
    from PyKDE4 import kdecore, kdeui
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *
    from PyQt4 import QtCore, QtGui
except ImportError:
    print("MKV Splitter requires PyQt4 and PyKDE4 in order to work, please make sure these are installed and try to run MKV Splitter again.")
    sys.exit(2)

try:
    from mydirrequester import *
except ImportError:
    print("mydirrequester seems to be missing from your installation. This is strange.")
    sys.exit(2)
try:
    from configuredialog import *
except ImportError:
    print("It seems the configuredialog file is missing from your installation. This is strange.")
try:
    from qtruncommandmkvsplitter import *
except ImportError:
    print("qtruncommandmkvsplitter seems to be missing from your installation. This is strange.")

try:
    from configobj import ConfigObj
except:
    print("configobj seems to be missing from your installation. This is strange.")
    
try:
    config = ConfigObj()
    config = ConfigObj('%s/.mkvsplitter.conf' % os.environ['HOME'])
except:
    pass
finally:
    default_close_after_split = False
    default_output_directory = os.environ['HOME']
    default_output_directory_is_source = True

    if 'default_close_after_split' in config:
        if config['default_close_after_split'] == 'False':
            default_close_after_split = False
    if 'default_output_directory' in config:
        default_output_directory = config['default_output_directory']
    if 'default_output_directory_is_source' in config:
        if config['default_output_directory_is_source'] == 'False':
            default_output_directory_is_source = False

class Splitter(KMainWindow):
    def handle_args(self):
        """ Handle the cli args here """
        try:
            opts, args = getopt.getopt(sys.argv[1:], "hi:", ["help","input-file="])

        except getopt.GetoptError:
            self.usage()
            self.close()
        self.input_file = ''
        for opt, arg in opts:
            if opt in ("-h", "--help"):
                self.usage()
                self.close()
            elif opt in ("-i",  "--input-file"):
                self.input_file = arg
            else:
                self.usage()
                self.close()
    def __del__(self):
            self.exiting = True
            print("MKVSplitter Main Thread exiting now.")

    def usage(self):
        print("mkv-splitter - splits mkv files by chapter names. PyQt4 GUI.\n-i --input-file=file - the input file.\n-h --help - print this message.")

    def __init__(self, *args):
        apply(KMainWindow.__init__,  (self,  ) + args)
        self.handle_args()

        self.title_name = ''
        self.window = QMainWindow()
        self.window.setMinimumSize(490, 530)
        self.window.setCentralWidget(self)
        self.main_layout_container = QWidget()
        self.main_layout = QVBoxLayout(self.main_layout_container)

        quit_action = QAction(self.loadIcon('window-close'), '&Quit', self)
        quit_action.setShortcuts(QKeySequence.Quit)
        quit_action.setStatusTip("Close this application.")
        quit_action.triggered.connect(self.quit)

        open_action = QAction(self.loadIcon('document-open-data'), '&Open', self)
        open_action.setShortcuts(QKeySequence.Open)
        open_action.setStatusTip("Open an MKV file to split")
        open_action.triggered.connect(self.open_file)

        configure_action = QAction(self.loadIcon('configure'), '&Configure...', self)
        configure_action.setStatusTip("Configure MKVSplitter")
        configure_action.triggered.connect(self.configure)

        self.menu_bar = QMenuBar()
        self.window.setMenuBar(self.menu_bar)
        self.help_menu = KHelpMenu(self,about_data)
        self.file_menu =        self.menu_bar.addMenu("&File")
        self.settings_menu =    self.menu_bar.addMenu("&Settings")
        self.help_menu =        self.menu_bar.addMenu(self.help_menu.menu())

        self.file_menu.addAction(open_action)
        self.file_menu.addAction(quit_action)

        self.settings_menu.addAction(configure_action)

        self.form_retainer = QWidget()
        self.form_layout = QFormLayout(self.form_retainer)
        self.form_layout.setFormAlignment(Qt.AlignTop)
        self.output_directory_edit = MyDirRequester(self)

        self.form_layout.addRow("Output Directory: ", self.output_directory_edit)

        self.output_file_basename_edit = QLineEdit(self)
        self.form_layout.addRow("Output Basename: ",  self.output_file_basename_edit)

        self.main_layout.addWidget(self.form_retainer)

        self.chapters_table = QTableWidget()
        self.chapters_table.setColumnCount(4)
        self.chapters_table.setShowGrid(True)
        self.chapters_table.horizontalHeader().setVisible(True)
        self.chapters_table.verticalHeader().setVisible(True)
        self.chapters_table.setHorizontalHeaderItem(0, QTableWidgetItem('Split'))
        self.chapters_table.setHorizontalHeaderItem(1, QTableWidgetItem('Start Time'))
        self.chapters_table.setHorizontalHeaderItem(2, QTableWidgetItem('Length'))
        self.chapters_table.setHorizontalHeaderItem(3, QTableWidgetItem('Chapter Name'))
        self.chapters_table.setSelectionMode(QAbstractItemView.SingleSelection)

        self.split_button = QPushButton()
        self.split_button.setText('Split')
        self.split_button.clicked.connect(self.perform_split)

        self.preview_button = QPushButton()
        self.preview_button.setText('Preview Chapter')
        self.preview_button.clicked.connect(self.preview_chapter)

        self.chapters_button_box_container = QWidget()
        self.chapters_button_box = QHBoxLayout(self.chapters_button_box_container)
        self.chapters_button_box.addWidget(self.preview_button)
        self.chapters_button_box.addWidget(self.split_button)

        self.main_layout.addWidget(self.chapters_table)
        self.main_layout.addWidget(self.chapters_button_box_container)

        self.progress_bar = QProgressBar()
        self.progress_bar.setValue(0)
        self.progress_bar.setVisible(False)

        self.main_layout.addWidget(self.progress_bar)
        self.setCentralWidget(self.main_layout_container)
        self.window.show()
        
        if not self.input_file == '': # No file selected yet
            self.preview_button.setEnabled(False)
            self.split_button.setEnabled(False)
            self.load_file()
    
    def open_file(self):
        self.input_file = KFileDialog.getOpenFileName(KUrl(), "*.mkv", self, "FileDialog")
        self.load_file()
    def load_file(self):
        print(default_output_directory_is_source)
        if default_output_directory_is_source:
            self.output_directory_edit.setText(os.path.dirname(str(self.input_file)))
        else:
            print("default is not source")
            self.output_directory_edit.setText(default_output_directory)
        
        self.output_file_basename_edit.setText(os.path.basename(str(self.input_file)).rsplit('.')[0])
        self.chapters = self.read_chapters()
        self.populate_chapters()
    def configure(self):
        c = ConfigureDialog(self)
        c.setWindowModality(Qt.WindowModal)
        c.exec_()

    def loadIcon(self, name):
        k = KIconLoader()
        return QIcon(k.loadIcon(name, KIconLoader.Small))

    def quit(self):
        self.close()

    def main(self):
        pass

    def populate_chapters(self):
        i = self.chapters_table.rowCount() - 1
        while i > -1:
            self.chapters_table.removeRow(i)
            i = i -1
        current = 0
        for chapter in self.chapters:
            self.chapters_table.insertRow(current)
            self.chapters_table.setCellWidget(current, 0, QCheckBox())
            self.chapters_table.setItem(current, 1, QTableWidgetItem(str(self.chapters[chapter]['start_time'])))
            self.chapters_table.setItem(current, 2, QTableWidgetItem(str(self.chapters[chapter]['length'])))
            self.chapters_table.setItem(current, 3, QTableWidgetItem(str(self.chapters[chapter]['chapter_name'])))
            self.chapters_table.resizeColumnsToContents()
            current = current + 1

    def read_chapters(self):
        lines = ''
        try:
            lines = run_command_with_output('mkvextract chapters -s \"%s\"' % (self.input_file))
        except:
            print("Command mkvextract exited with a non-zero exit status of %d." % status)
            return Null
        finally:
            lines = lines.split('\n')
            lines2 = []
            chapters = {}
            for line in lines:
                line = line.strip()
                if len(line) > 1:
                    lines2.append(line.strip())
            chapter_number = 1
            time_line = True
            for line in lines:
                right = line.split('=')[1]
                if time_line == True:
                    chapters[chapter_number] = dict()
                    chapters[chapter_number]['start_time'] = right
                    time_line = False
                else:
                    chapters[chapter_number]['chapter_name'] = right

                    time_line = True
                    chapter_number = chapter_number + 1

            for x in range(len(chapters)):
                if not x+1 == len(chapters):
                    chapters[x+1]['length'] = self.time_subtraction(chapters[x+1]['start_time'], chapters[x+2]['start_time'])
                else:
                    end_time = run_command_with_output("mkvinfo \"%s\" | grep Duration | cut -d '(' -f 2 | cut -d ')' -f 1" % self.input_file)
                    chapters[x+1]['length'] = self.time_subtraction(chapters[x+1]['start_time'], end_time)

            return chapters

    def time_subtraction(self, first, second):
        #Second-minus-First
        [first_hours,  first_minutes,  first_seconds] = first.split(':')
        [second_hours,  second_minutes,  second_seconds] = second.split(':')
        first_hours = int(first_hours)
        second_hours = int(second_hours)
        first_minutes = int(first_minutes)
        second_minutes = int(second_minutes)
        first_seconds = float(first_seconds)
        second_seconds = float(second_seconds)

        if second_seconds >= first_seconds:
            result_seconds = second_seconds - first_seconds
        else:
            second_minutes = second_minutes - 1
            second_seconds = second_seconds + 60
            result_seconds = second_seconds - first_seconds

        if second_minutes >= first_minutes:
            result_minutes = second_minutes - first_minutes
        else:
            second_hours = second_hours - 1
            second_minutes = second_minutes + 60
            result_minutes = second_minutes - first_minutes

        result_hours = second_hours - first_hours

        result_hours = str(result_hours).zfill(2)
        result_minutes = str(result_minutes).zfill(2)
        result_seconds = str("%0.3f" % result_seconds).zfill(6)

        result = "%s:%s:%s" % (result_hours, result_minutes, result_seconds)

        return result

    def preview_chapter(self):
        current_row = self.chapters_table.currentRow()
        run_command("mplayer -chapter %s-%s \"%s\"" % (current_row+1,  current_row+1,  self.input_file))

    def update_progress(self, progress):
        self.progress_bar.setValue(int(progress))
        if int(progress) == 100:
            if not default_close_on_exit:
                self.progress_bar.setVisible(True)
                self.menu_bar.setEnabled(True)
                self.form_layout.setEnabled(True)
                self.preview_button.setEnabled(True)
                self.split_button.setEnabled(True)
                self.output_file_basename_edit.setEnabled(True)
                self.output_directory_edit.setEnabled(True)
                self.chapters_table.setEnabled(True)
                self.setCursor(Qt.ArrowCursor)
            else:
                sys.exit(0)

    def perform_split(self):
        self.progress_bar.setVisible(True)
        self.menu_bar.setEnabled(False)
        self.form_layout.setEnabled(False)
        self.preview_button.setEnabled(False)
        self.split_button.setEnabled(False)
        self.output_file_basename_edit.setEnabled(False)
        self.output_directory_edit.setEnabled(False)
        self.chapters_table.setEnabled(False)
        self.setCursor(Qt.WaitCursor)

        split_points = []
        for row in range(self.chapters_table.rowCount()):
            if self.chapters_table.cellWidget(row, 0).isChecked():
                split_points.append(str(self.chapters_table.item(row, 1).text()))

        split_points = ','.join(split_points)
        outdir = self.output_directory_edit.url()
        basename = self.output_file_basename_edit.text()


        command = 'mkvmerge -o "%s/%s.mkv" --split "timecodes:%s" "%s" 2>&1' % (outdir,  basename,  split_points,  self.input_file)
        run_command_progress(command, self.update_progress)

app_name = "mkv-splitter"
catalog = ""
program_name = ki18n("mkv-splitter")
version = "0.3.2"
description = ki18n("MKVSplitter is a program to split MKV files by their chapter information")
license = KAboutData.License_GPL
copyright = ki18n("(c) 2011 Dustin Widmann")
text = ki18n("")
home_page = ""
bug_email = "1m.0n.f1r3@gmail.com"

about_data = KAboutData(app_name, catalog, program_name, version, description, license, copyright, text, home_page, bug_email)

KCmdLineArgs.init([sys.argv[0]] , about_data)
app=KApplication()
splitter = Splitter()
splitter.show()
app.connect(app, SIGNAL("lastWindowClosed()"),  app,  SLOT("quit()"))
sys.exit(app.exec_())
